package com.jiuqi.util;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.Comparator;
import java.util.Iterator;
import java.util.List;
import java.util.StringTokenizer;

public class StringUtils {
   public static final String LINE_SEPARATOR = System.getProperty("line.separator", "\r\n");

   private StringUtils() {
   }

   public static String clean(String str) {
      return str == null ? "" : str.trim();
   }

   public static String trim(String str) {
      return str == null ? null : str.trim();
   }

   public static String deleteWhitespace(String str) {
      StringBuffer buffer = new StringBuffer();
      int sz = str.length();

      for(int i = 0; i < sz; ++i) {
         if (!Character.isWhitespace(str.charAt(i))) {
            buffer.append(str.charAt(i));
         }
      }

      return buffer.toString();
   }

   public static boolean isNotEmpty(String str) {
      return str != null && str.length() > 0;
   }

   public static boolean isEmpty(String str) {
      return str == null || str.trim().length() == 0;
   }

   public static boolean equals(String str1, String str2) {
      return str1 == null ? str2 == null : str1.equals(str2);
   }

   public static boolean equalsIgnoreCase(String str1, String str2) {
      return str1 == null ? str2 == null : str1.equalsIgnoreCase(str2);
   }

   public static int indexOfAny(String str, String[] searchStrs) {
      if (str != null && searchStrs != null) {
         int sz = searchStrs.length;
         int ret = Integer.MAX_VALUE;
         int tmp = false;

         for(int i = 0; i < sz; ++i) {
            int tmp = str.indexOf(searchStrs[i]);
            if (tmp != -1 && tmp < ret) {
               ret = tmp;
            }
         }

         return ret == Integer.MAX_VALUE ? -1 : ret;
      } else {
         return -1;
      }
   }

   public static int lastIndexOfAny(String str, String[] searchStrs) {
      if (str != null && searchStrs != null) {
         int sz = searchStrs.length;
         int ret = -1;
         int tmp = false;

         for(int i = 0; i < sz; ++i) {
            int tmp = str.lastIndexOf(searchStrs[i]);
            if (tmp > ret) {
               ret = tmp;
            }
         }

         return ret;
      } else {
         return -1;
      }
   }

   public static String substring(String str, int start) {
      if (str == null) {
         return null;
      } else {
         if (start < 0) {
            start += str.length();
         }

         if (start < 0) {
            start = 0;
         }

         return start > str.length() ? "" : str.substring(start);
      }
   }

   public static String substring(String str, int start, int end) {
      if (str == null) {
         return null;
      } else {
         if (end < 0) {
            end += str.length();
         }

         if (start < 0) {
            start += str.length();
         }

         if (end > str.length()) {
            end = str.length();
         }

         if (start > end) {
            return "";
         } else {
            if (start < 0) {
               start = 0;
            }

            if (end < 0) {
               end = 0;
            }

            return str.substring(start, end);
         }
      }
   }

   public static String left(String str, int len) {
      if (len < 0) {
         throw new IllegalArgumentException("Requested String length " + len + " is less than zero");
      } else {
         return str != null && str.length() > len ? str.substring(0, len) : str;
      }
   }

   public static String right(String str, int len) {
      if (len < 0) {
         throw new IllegalArgumentException("Requested String length " + len + " is less than zero");
      } else {
         return str != null && str.length() > len ? str.substring(str.length() - len) : str;
      }
   }

   public static String mid(String str, int pos, int len) {
      if (pos >= 0 && (str == null || pos <= str.length())) {
         if (len < 0) {
            throw new IllegalArgumentException("Requested String length " + len + " is less than zero");
         } else if (str == null) {
            return null;
         } else {
            return str.length() <= pos + len ? str.substring(pos) : str.substring(pos, pos + len);
         }
      } else {
         throw new StringIndexOutOfBoundsException("String index " + pos + " is out of bounds");
      }
   }

   public static String[] split(String str) {
      return split(str, (String)null, -1);
   }

   public static String[] splitWithSpaceElement(String text, String separator) {
      int pos = text.indexOf(separator);

      int count;
      for(count = 0; pos != -1; pos = text.indexOf(separator, pos + 1)) {
         ++count;
      }

      String[] sReturn = new String[count + 1];
      pos = text.indexOf(separator);
      count = 0;

      int prepos;
      for(prepos = -1; pos != -1; ++count) {
         sReturn[count] = text.substring(prepos + 1, pos);
         prepos = pos;
         pos = text.indexOf(separator, pos + 1);
      }

      if (prepos + 1 == text.length()) {
         sReturn[sReturn.length - 1] = "";
      } else {
         sReturn[sReturn.length - 1] = text.substring(prepos + 1);
      }

      return sReturn;
   }

   public static String[] split(String text, String separator) {
      return split(text, separator, -1);
   }

   public static String[] split(String str, String separator, int max) {
      StringTokenizer tok = null;
      if (separator == null) {
         tok = new StringTokenizer(str);
      } else {
         tok = new StringTokenizer(str, separator);
      }

      int listSize = tok.countTokens();
      if (max > 0 && listSize > max) {
         listSize = max;
      }

      String[] list = new String[listSize];
      int i = 0;
      int lastTokenBegin = false;

      for(int lastTokenEnd = 0; tok.hasMoreTokens(); ++i) {
         int lastTokenBegin;
         if (max > 0 && i == listSize - 1) {
            String endToken = tok.nextToken();
            lastTokenBegin = str.indexOf(endToken, lastTokenEnd);
            list[i] = str.substring(lastTokenBegin);
            break;
         }

         list[i] = tok.nextToken();
         lastTokenBegin = str.indexOf(list[i], lastTokenEnd);
         lastTokenEnd = lastTokenBegin + list[i].length();
      }

      return list;
   }

   public static String concatenate(Object[] array) {
      return join(array, "");
   }

   public static String join(Object[] array, String separator) {
      if (separator == null) {
         separator = "";
      }

      int arraySize = array.length;
      int bufSize = arraySize == 0 ? 0 : (array[0].toString().length() + separator.length()) * arraySize;
      StringBuffer buf = new StringBuffer(bufSize);

      for(int i = 0; i < arraySize; ++i) {
         if (i > 0) {
            buf.append(separator);
         }

         buf.append(array[i]);
      }

      return buf.toString();
   }

   public static String join(Iterator iterator, String separator) {
      if (separator == null) {
         separator = "";
      }

      StringBuffer buf = new StringBuffer(256);

      while(iterator.hasNext()) {
         buf.append(iterator.next());
         if (iterator.hasNext()) {
            buf.append(separator);
         }
      }

      return buf.toString();
   }

   public static String replaceOnce(String text, String repl, String with) {
      return replace(text, repl, with, 1);
   }

   public static String replace(String text, String repl, String with) {
      return replace(text, repl, with, -1);
   }

   public static String replace(String text, String repl, String with, int max) {
      if (text == null) {
         return null;
      } else if (repl != null && repl.length() != 0) {
         StringBuffer buf = new StringBuffer(text.length());
         int start = 0;
         int end = false;

         int end;
         while((end = text.indexOf(repl, start)) != -1) {
            buf.append(text.substring(start, end)).append(with);
            start = end + repl.length();
            --max;
            if (max == 0) {
               break;
            }
         }

         buf.append(text.substring(start));
         return buf.toString();
      } else {
         return text;
      }
   }

   public static String overlayString(String text, String overlay, int start, int end) {
      return (new StringBuffer(start + overlay.length() + text.length() - end + 1)).append(text.substring(0, start)).append(overlay).append(text.substring(end)).toString();
   }

   public static String center(String str, int size) {
      return center(str, size, " ");
   }

   public static String center(String str, int size, String delim) {
      int sz = str.length();
      int p = size - sz;
      if (p < 1) {
         return str;
      } else {
         str = leftPad(str, sz + p / 2, delim);
         str = rightPad(str, size, delim);
         return str;
      }
   }

   public static String chomp(String str) {
      return chomp(str, "\n");
   }

   public static String chomp(String str, String sep) {
      int idx = str.lastIndexOf(sep);
      return idx != -1 ? str.substring(0, idx) : str;
   }

   public static String chompLast(String str) {
      return chompLast(str, "\n");
   }

   public static String chompLast(String str, String sep) {
      if (str.length() == 0) {
         return str;
      } else {
         String sub = str.substring(str.length() - sep.length());
         return sep.equals(sub) ? str.substring(0, str.length() - sep.length()) : str;
      }
   }

   public static String getChomp(String str, String sep) {
      int idx = str.lastIndexOf(sep);
      if (idx == str.length() - sep.length()) {
         return sep;
      } else {
         return idx != -1 ? str.substring(idx) : "";
      }
   }

   public static String prechomp(String str, String sep) {
      int idx = str.indexOf(sep);
      return idx != -1 ? str.substring(idx + sep.length()) : str;
   }

   public static String getPrechomp(String str, String sep) {
      int idx = str.indexOf(sep);
      return idx != -1 ? str.substring(0, idx + sep.length()) : "";
   }

   public static String chop(String str) {
      if ("".equals(str)) {
         return "";
      } else if (str.length() == 1) {
         return "";
      } else {
         int lastIdx = str.length() - 1;
         String ret = str.substring(0, lastIdx);
         char last = str.charAt(lastIdx);
         return last == '\n' && ret.charAt(lastIdx - 1) == '\r' ? ret.substring(0, lastIdx - 1) : ret;
      }
   }

   public static String chopNewline(String str) {
      int lastIdx = str.length() - 1;
      char last = str.charAt(lastIdx);
      if (last == '\n') {
         if (str.charAt(lastIdx - 1) == '\r') {
            --lastIdx;
         }
      } else {
         ++lastIdx;
      }

      return str.substring(0, lastIdx);
   }

   public static String escape(String str) {
      int sz = str.length();
      StringBuffer buffer = new StringBuffer(2 * sz);

      for(int i = 0; i < sz; ++i) {
         char ch = str.charAt(i);
         if (ch > 4095) {
            buffer.append("\\u" + Integer.toHexString(ch));
         } else if (ch > 255) {
            buffer.append("\\u0" + Integer.toHexString(ch));
         } else if (ch > 127) {
            buffer.append("\\u00" + Integer.toHexString(ch));
         } else if (ch < ' ') {
            switch (ch) {
               case '\b':
                  buffer.append('\\');
                  buffer.append('b');
                  break;
               case '\t':
                  buffer.append('\\');
                  buffer.append('t');
                  break;
               case '\n':
                  buffer.append('\\');
                  buffer.append('n');
                  break;
               case '\u000b':
               default:
                  if (ch > 15) {
                     buffer.append("\\u00" + Integer.toHexString(ch));
                  } else {
                     buffer.append("\\u000" + Integer.toHexString(ch));
                  }
                  break;
               case '\f':
                  buffer.append('\\');
                  buffer.append('f');
                  break;
               case '\r':
                  buffer.append('\\');
                  buffer.append('r');
            }
         } else {
            switch (ch) {
               case '"':
                  buffer.append('\\');
                  buffer.append('"');
                  break;
               case '\'':
                  buffer.append('\\');
                  buffer.append('\'');
                  break;
               case '\\':
                  buffer.append('\\');
                  buffer.append('\\');
                  break;
               default:
                  buffer.append(ch);
            }
         }
      }

      return buffer.toString();
   }

   public static String repeat(String str, int repeat) {
      StringBuffer buffer = new StringBuffer(repeat * str.length());

      for(int i = 0; i < repeat; ++i) {
         buffer.append(str);
      }

      return buffer.toString();
   }

   public static String rightPad(String str, int size) {
      return rightPad(str, size, " ");
   }

   public static String rightPad(String str, int size, String delim) {
      size = (size - str.length()) / delim.length();
      if (size > 0) {
         str = str + repeat(delim, size);
      }

      return str;
   }

   public static String leftPad(String str, int size) {
      return leftPad(str, size, " ");
   }

   public static String leftPad(String str, int size, String delim) {
      size = (size - str.length()) / delim.length();
      if (size > 0) {
         str = repeat(delim, size) + str;
      }

      return str;
   }

   public static String strip(String str) {
      return strip(str, (String)null);
   }

   public static String strip(String str, String delim) {
      str = stripStart(str, delim);
      return stripEnd(str, delim);
   }

   public static String[] stripAll(String[] strs) {
      return stripAll(strs, (String)null);
   }

   public static String[] stripAll(String[] strs, String delimiter) {
      if (strs != null && strs.length != 0) {
         int sz = strs.length;
         String[] newArr = new String[sz];

         for(int i = 0; i < sz; ++i) {
            newArr[i] = strip(strs[i], delimiter);
         }

         return newArr;
      } else {
         return strs;
      }
   }

   public static String stripEnd(String str, String strip) {
      if (str == null) {
         return null;
      } else {
         int end = str.length();
         if (strip == null) {
            while(end != 0 && Character.isWhitespace(str.charAt(end - 1))) {
               --end;
            }
         } else {
            while(end != 0 && strip.indexOf(str.charAt(end - 1)) != -1) {
               --end;
            }
         }

         return str.substring(0, end);
      }
   }

   public static String stripStart(String str, String strip) {
      if (str == null) {
         return null;
      } else {
         int start = 0;
         int sz = str.length();
         if (strip == null) {
            while(start != sz && Character.isWhitespace(str.charAt(start))) {
               ++start;
            }
         } else {
            while(start != sz && strip.indexOf(str.charAt(start)) != -1) {
               ++start;
            }
         }

         return str.substring(start);
      }
   }

   public static String upperCase(String str) {
      return str == null ? null : str.toUpperCase();
   }

   public static String lowerCase(String str) {
      return str == null ? null : str.toLowerCase();
   }

   public static String uncapitalise(String str) {
      if (str == null) {
         return null;
      } else {
         return str.length() == 0 ? "" : (new StringBuffer(str.length())).append(Character.toLowerCase(str.charAt(0))).append(str.substring(1)).toString();
      }
   }

   public static String capitalise(String str) {
      if (str == null) {
         return null;
      } else {
         return str.length() == 0 ? "" : (new StringBuffer(str.length())).append(Character.toTitleCase(str.charAt(0))).append(str.substring(1)).toString();
      }
   }

   public static String swapCase(String str) {
      if (str == null) {
         return null;
      } else {
         int sz = str.length();
         StringBuffer buffer = new StringBuffer(sz);
         boolean whitespace = false;
         char ch = false;
         char tmp = false;

         for(int i = 0; i < sz; ++i) {
            char ch = str.charAt(i);
            char tmp;
            if (Character.isUpperCase(ch)) {
               tmp = Character.toLowerCase(ch);
            } else if (Character.isTitleCase(ch)) {
               tmp = Character.toLowerCase(ch);
            } else if (Character.isLowerCase(ch)) {
               if (whitespace) {
                  tmp = Character.toTitleCase(ch);
               } else {
                  tmp = Character.toUpperCase(ch);
               }
            } else {
               tmp = ch;
            }

            buffer.append(tmp);
            whitespace = Character.isWhitespace(ch);
         }

         return buffer.toString();
      }
   }

   public static String capitaliseAllWords(String str) {
      if (str == null) {
         return null;
      } else {
         int sz = str.length();
         StringBuffer buffer = new StringBuffer(sz);
         boolean space = true;

         for(int i = 0; i < sz; ++i) {
            char ch = str.charAt(i);
            if (Character.isWhitespace(ch)) {
               buffer.append(ch);
               space = true;
            } else if (space) {
               buffer.append(Character.toTitleCase(ch));
               space = false;
            } else {
               buffer.append(ch);
            }
         }

         return buffer.toString();
      }
   }

   public static String getNestedString(String str, String tag) {
      return getNestedString(str, tag, tag);
   }

   public static String getNestedString(String str, String open, String close) {
      if (str == null) {
         return null;
      } else {
         int start = str.indexOf(open);
         if (start != -1) {
            int end = str.indexOf(close, start + open.length());
            if (end != -1) {
               return str.substring(start + open.length(), end);
            }
         }

         return null;
      }
   }

   public static int countMatches(String str, String sub) {
      if (str == null) {
         return 0;
      } else {
         int count = 0;

         for(int idx = 0; (idx = str.indexOf(sub, idx)) != -1; idx += sub.length()) {
            ++count;
         }

         return count;
      }
   }

   public static boolean isAlpha(String str) {
      if (str == null) {
         return false;
      } else {
         int sz = str.length();

         for(int i = 0; i < sz; ++i) {
            if (!Character.isLetter(str.charAt(i))) {
               return false;
            }
         }

         return true;
      }
   }

   public static boolean isAlphaSpace(String str) {
      if (str == null) {
         return false;
      } else {
         int sz = str.length();

         for(int i = 0; i < sz; ++i) {
            if (!Character.isLetter(str.charAt(i)) && str.charAt(i) != ' ') {
               return false;
            }
         }

         return true;
      }
   }

   public static boolean isAlphanumeric(String str) {
      if (str == null) {
         return false;
      } else {
         int sz = str.length();

         for(int i = 0; i < sz; ++i) {
            if (!Character.isLetterOrDigit(str.charAt(i))) {
               return false;
            }
         }

         return true;
      }
   }

   public static boolean isAlphanumericSpace(String str) {
      if (str == null) {
         return false;
      } else {
         int sz = str.length();

         for(int i = 0; i < sz; ++i) {
            if (!Character.isLetterOrDigit(str.charAt(i)) && str.charAt(i) != ' ') {
               return false;
            }
         }

         return true;
      }
   }

   public static boolean isNumeric(String str) {
      if (str == null) {
         return false;
      } else {
         int sz = str.length();

         for(int i = 0; i < sz; ++i) {
            if (!Character.isDigit(str.charAt(i))) {
               return false;
            }
         }

         return true;
      }
   }

   public static boolean isNumericSpace(String str) {
      if (str == null) {
         return false;
      } else {
         int sz = str.length();

         for(int i = 0; i < sz; ++i) {
            if (!Character.isDigit(str.charAt(i)) && str.charAt(i) != ' ') {
               return false;
            }
         }

         return true;
      }
   }

   public static String defaultString(String str) {
      return defaultString(str, "");
   }

   public static String defaultString(String str, String defaultString) {
      return str == null ? defaultString : str;
   }

   public static String reverse(String str) {
      return str == null ? null : (new StringBuffer(str)).reverse().toString();
   }

   public static String reverseDelimitedString(String str, String delimiter) {
      String[] strs = split(str, delimiter);
      reverseArray(strs);
      return join((Object[])strs, delimiter);
   }

   private static void reverseArray(Object[] array) {
      int i = 0;

      for(int j = array.length - 1; j > i; ++i) {
         Object tmp = array[j];
         array[j] = array[i];
         array[i] = tmp;
         --j;
      }

   }

   public static boolean containsOnly(String str, char[] valid) {
      if (str != null && valid != null) {
         int strSize = str.length();
         int validSize = valid.length;

         for(int i = 0; i < strSize; ++i) {
            boolean contains = false;

            for(int j = 0; j < validSize; ++j) {
               if (valid[j] == str.charAt(i)) {
                  contains = true;
                  break;
               }
            }

            if (!contains) {
               return false;
            }
         }

         return true;
      } else {
         return false;
      }
   }

   public static String encodeStringArray(String[] strs) {
      if (strs == null) {
         return "";
      } else {
         StringBuffer sb = new StringBuffer();

         for(int i = 0; i < strs.length; ++i) {
            sb.append(strs[i]);
            sb.append(";");
         }

         return sb.toString();
      }
   }

   public static String[] decodeStringArray(String str) {
      if (str == null) {
         return null;
      } else {
         StringTokenizer st = new StringTokenizer(str, ";");
         List l = new ArrayList();

         while(st.hasMoreElements()) {
            l.add(st.nextElement());
         }

         String[] ret = new String[l.size()];

         for(int i = 0; i < l.size(); ++i) {
            ret[i] = (String)l.get(i);
         }

         return ret;
      }
   }

   public static int lengthOf(String s) {
      if (s == null) {
         return -1;
      } else {
         int size = s.length();
         int ret = 0;

         for(int i = 0; i < size; ++i) {
            ret += s.charAt(i) > 255 ? 2 : 1;
         }

         return ret;
      }
   }

   public static int getMaxLenOfStrList(String[] strArr) {
      List list = Arrays.asList(strArr);
      int ret = getMaxLenOfStrList(list);
      return ret;
   }

   public static int getMaxLenOfStrList(List strList) {
      class StrComparator implements Comparator {
         public boolean equals(Object object) {
            return false;
         }

         public int compare(Object obj1, Object obj2) {
            if (obj1 != null && obj2 != null) {
               return StringUtils.lengthOf((String)obj1) - StringUtils.lengthOf((String)obj2);
            } else {
               return obj1 == obj2 ? 0 : (obj2 == null ? 1 : -1);
            }
         }
      }

      StrComparator c = new StrComparator();
      String retStr = (String)Collections.max(strList, c);
      c = null;
      return lengthOf(retStr);
   }

   public static int indexOf(char c, String src) {
      int len = src.length();

      for(int i = 0; i < len; ++i) {
         if (src.charAt(i) == c) {
            return i;
         }
      }

      return -1;
   }

   public static String quote(String s, char quote) {
      if (isEmpty(s)) {
         return s;
      } else {
         StringBuffer retStr = new StringBuffer();
         retStr.append(quote);

         for(int i = 0; i < s.length(); ++i) {
            if (s.charAt(i) == quote) {
               retStr.append(quote);
               retStr.append(quote);
            } else {
               retStr.append(s.charAt(i));
            }
         }

         retStr.append(quote);
         return retStr.toString();
      }
   }

   public static String dequote(String s, char quote) {
      if (isEmpty(s)) {
         return s;
      } else if (s.charAt(0) != quote) {
         return null;
      } else {
         StringBuffer retStr = new StringBuffer(s.length());
         int i = 1;

         while(i < s.length()) {
            char curCh = s.charAt(i++);
            if (curCh == quote) {
               if (i >= s.length()) {
                  break;
               }

               char nextCh = s.charAt(i++);
               if (nextCh != quote) {
                  break;
               }

               retStr.append(quote);
            } else {
               retStr.append(s.charAt(i));
            }
         }

         return retStr.toString();
      }
   }

   public static String eliminate(String s, char ch) {
      if (s == null) {
         return null;
      } else {
         int next = s.indexOf(ch);
         if (next == -1) {
            return s;
         } else {
            StringBuffer buffer = new StringBuffer(s.length());
            int current = 0;

            while(current < s.length()) {
               buffer.append(s.substring(current, next));
               current = next + 1;
               next = s.indexOf(ch, current);
               if (next == -1) {
                  next = s.length();
               }
            }

            return buffer.toString();
         }
      }
   }
}
